/*********************************************************
 * Filename: main.h
 * 
 * Purpose:
 * Provide the user with an easy-to-use GUI to control the
 * GT-VP's internal settings and view module data. 
 * 
 * Version: 4.0 (ME-N63-05)
 * Date: 8/3/2021
 * Website: noritake-elec.com
 * (C) 2021 Noritake Co., Inc.
 *********************************************************/

#ifndef MAIN_H
#define MAIN_H

#include "GT1P.h"
#include "GT1P_USB_LIB.h"

//GT-VP(1P) Memory switch definitions
#define BRIGHTNESS_MS 5
#define I2C_HID_SLAVE_ADDRESS_MS 46
#define I2C_COMMAND_SLAVE_ADDRESS_MS 47
#define UART_BAUDRATE_MS 48
#define UART_PARITY_MS 49
#define THRESHOLD_MS 59
#define TOUCH_SENS_SELECT_MS 62
#define TSP_SELECT_MS 63

// Maximum filename character to display
// (including ellipsis and null terminator)
#define FILENAME_DISPLAY_MAX 29

// TSP file compatible GT-VP firmware version for 7" and 5" modules
#define TSP_FW 150
// TSP file compatible GT-VP firmware version for 10.1" modules
#define TSP_FW_101 156

//TSP file type sizes
//NOTE: Type-B size is still subject to change
#define TSP_TYPE_A_SIZE 1152
#define TSP_TYPE_B_SIZE 2048

//Product ID definitions
#define GTWV050VHB00P 0x12dd
#define GTWS070VHA00P 0x12d0
#define GT800X480A_1303P 0x12de
#define GT1280X800A_1303P 0x12df

// Struct to store TSP file header information
struct TSP_File {
	char fileID[3];
	char packageID[5];
	char author[17];
	char formatType[2];
	char productInfo[33];
	char specCode[10];
	char notes[65];
	unsigned char data[1024];
};
struct TSP_File openedTSPFile;

//Struct to store connected module information
struct TFT_Info {
	unsigned char boot[7];
	unsigned char firm[7];
	unsigned char prod[18];
	unsigned char width[7];
	unsigned char height[7];
	unsigned char memCheck[7];
	unsigned char TSPName[18];
	unsigned char TSPID[4];
};
struct TFT_Info displayInfo;

// Baudrate flag values
enum baudrateSelection {
	BR_38400_Default = 0,
	BR_4800 = 1,
	BR_9600 = 2,
	BR_19200 = 3,
	BR_38400 = 4,
	BR_57600 = 5,
	BR_115200 = 6
};

// Parity flag values
enum paritySelection {
	parityNone = 0,
	parityEven = 1,
	parityOdd = 2
};

// TSP file positions
enum TSPPosition {
	factoryDefault = 0,
	TSPPosition1 = 1,
	TSPPosition2 = 2,
	TSPPosition3 = 3,
	TSPPosition4 = 4
};

/**
 * New PN = 0
 * Old PN = 1
 */
uint8_t oldPNFlag = 0;

uint8_t compatFWFlag = 0;

//Global GTK Object Pointers
GtkAdjustment *brightnessAdjustment;
GtkAdjustment *thresholdAdjustment;
GtkAdjustment *I2CHIDAdjustment;
GtkAdjustment *I2CCommandAdjustment;
GtkAdjustment *acceptableNoiseAdjustment;
GtkAdjustment *measurementNumAdjustment;
GtkAdjustment *scrollAdjustment;

GtkDialog     *brightnessWarningWindow;
GtkDialog 	  *thresholdWarningWindow;
GtkDialog     *brightnessSaveWindow;
GtkDialog     *thresholdSaveWindow;
GtkDialog     *brightnessFileWindow;
GtkDialog     *thresholdFileWindow;
GtkDialog     *touchPackageChooserDialog;
GtkDialog	  *TSPFileCompatDialog;
GtkDialog	  *TSPFileVerifyDialog;
GtkDialog	  *ConnectFailedDialog;

GtkLabel      *brightnessSaveWindowLabel;
GtkLabel      *brightnessSaveWindowLabel2;
GtkLabel      *thresholdSaveWindowLabel;
GtkLabel      *thresholdSaveWindowLabel2;
GtkLabel      *bootLabel;
GtkLabel	  *firmwareLabel;
GtkLabel	  *productTypeLabel;
GtkLabel	  *resolutionLabel;
GtkLabel	  *memoryChecksumLabel;
GtkLabel	  *connectionStatusLabel;
GtkLabel	  *TSP_NameLabel;
GtkLabel	  *TSP_IDLabel;
GtkLabel	  *TSP_SelectionLabel;
GtkLabel	  *TSRResultsExplanationLabel;
GtkLabel	  *displayMenuButtonLabel;
GtkLabel      *TSP_FileLabel;
GtkLabel      *TSP0_Label;
GtkLabel      *TSP1_Label;
GtkLabel      *TSP2_Label;
GtkLabel      *TSP3_Label;
GtkLabel      *TSP4_Label;
GtkLabel	  *touchMonitorHelpLabel;
GtkLabel	  *baudrateMenuLabel;
GtkLabel	  *parityMenuLabel;

GtkListStore  *displayList;

GtkWidget     *displayCombobox;
GtkWidget	  *touchMonitorCombobox;
GtkWidget	  *TouchMonitorMenubutton;
GtkWidget	  *startTouchMonitorButton;
GtkWidget	  *stopTouchMonitorButton;
GtkWidget	  *runTSRTestButton;
GtkWidget	  *readTSRStateButton;
GtkWidget	  *acceptableNoiseSpinbutton;
GtkWidget	  *measurementNumSpinbutton;
GtkWidget	  *TSR_spinner;
GtkWidget	  *touchLevelToolButton;

GtkWidget	  *touchMonitorTextview;
GtkWidget	  *touchMonitorScrolledWindow;
GtkTextBuffer *touchMonitorTextbuffer;

GtkSwitch	  *TSRSwitch;

GtkTextMark	  *mark;
GtkTextIter	  iter;
GtkNotebook	  *controlcenterNotebook;

// Touch Setting Package Tab
GtkWidget 	  *TSP0_RadioButton;
GtkWidget     *TSP1_RadioButton;
GtkWidget     *TSP2_RadioButton;
GtkWidget     *TSP3_RadioButton;
GtkWidget     *TSP4_RadioButton;
GtkWidget	  *openTouchSettingPackageButton;
GtkWidget	  *loadTouchSettingPackageButton;
GtkWidget	  *selectTouchSettingPackageButton;
GtkWidget	  *saveTouchSettingPackageSelectionButton;

void initObjects(GtkBuilder *builder);
void on_window_main_destroy();
void getProductInfo();
void brightnessButton_clicked_cb();
void brightnessYesButton_clicked_cb();
void brightnessNoButton_clicked_cb();
void brightnessSaveYesButton_clicked_cb();
void on_brightnessSaveNoButton_clicked();
void thresholdSaveYesButton_clicked_cb();
void thresholdSaveNoButton_clicked_cb();
void thresholdButton_clicked_cb();
void thresholdWarningYesButton_clicked_cb();
void thresholdWarningNoButton_clicked_cb();
void factoryDefaultButton_clicked_cb();
void saveBrightnessButton_clicked_cb();
void saveThresholdButton_clicked_cb();
void brightnessFileDialogOkButton_clicked_cb();
void thresholdFileDialogOkButton_clicked_cb();
void touchMonitorSelect();
void touchMonitor_OFF();
void touchMonitor_Single();
void touchMonitor_Multi();
void scrollTextview(char * stringToAdd);
void *startTouchMonitorThread();
void startTouchMonitorButton_clicked_cb();
void stopTouchMonitorButton_clicked_cb();
void readAllMSButton_clicked_cb();
void writeAllMSButton_clicked_cb();
void defaultMSButton_clicked_cb();
void exportMSButton_clicked_cb();
void runTSRTestButton_clicked_cb();
void TSRSwitch_state_set_cb();
void readTSRStateButton_clicked_cb();
void GTWV050VHB00P_selected();
void GTWS070VHA00P_selected();
void GT800X480A_1303P_selected();
void GT1280X800A_1303P_selected();
void GTWX101VHB00P_selected();
void connectFailedDialogOkButton_clicked_cb();
void enableTSRWidgets(gboolean setting);
void enableTSPWidgets(gboolean setting);
void on_TSP_RadioButton_clicked();
void openTouchPackageButton_clicked_cb();
void TSPFileVerifyDialogOkButton_clicked_cb();
void TSPFileCompatDialogOkButton_clicked_cb();
void closeTouchPackageButton_clicked_cb();
void openTouchPackageButton_clicked_cb();
void loadTouchSettingPackageButton_clicked_cb();
void sendTSP(uint8_t location, GtkLabel *label, char *TSPfilename, const gchar *filename);
void selectTouchSettingPackageButton_clicked_cb();
void resetTSPTab();
void saveTouchSettingPackageSelectionButton_clicked_cb();
void BR_38400bps_Default_selected();
void BR_4800bps_selected();
void BR_9600bps_selected();
void BR_19200bps_selected();
void BR_38400bps_selected();
void BR_57600bps_selected();
void BR_115200bps_selected();
void Parity_None_selected();
void Parity_Even_selected();
void Parity_Odd_selected();
void touchLevelToolButton_clicked_cb();

#endif
